
INLINE_ANCHOR_DEFINE_FEATURE_NAME = 'tour_guide.inline_anchor_define'
INLINE_ANCHOR_DEFINE_ENTITY_TYPE = 'tour_guide.INLINE_ANCHOR_DEFINE'

INLINE_ANCHOR_LINK_FEATURE_NAME = 'tour_guide.inline_anchor_link'
INLINE_ANCHOR_LINK_ENTITY_TYPE = 'tour_guide.INLINE_ANCHOR_LINK'

INLINE_ANCHOR_LABELLED_LINK_FEATURE_NAME = 'tour_guide.inline_anchor_labelled_link'
INLINE_ANCHOR_LABELLED_LINK_ENTITY_TYPE = 'tour_guide.INLINE_ANCHOR_LABELLED_LINK'

feature_names = [INLINE_ANCHOR_DEFINE_FEATURE_NAME,
                 INLINE_ANCHOR_LINK_FEATURE_NAME,
                 INLINE_ANCHOR_LABELLED_LINK_FEATURE_NAME
                 ]
