@charset "UTF-8";

.rich-link-block {
    display: flex;
    flex-flow: row wrap;
    position: relative;
}

.rich-link-block > [data-contentpath="rich_text"],
.rich-link-block > [data-contentpath="external_url"] {
    width: 100%;
    flex: 1 0 auto;
}

.rich-link-block > [data-contentpath="page"],
.rich-link-block > [data-contentpath="page_fragment"] {
    flex: 1 1 auto;
}
