.tour_guide_anchor{-webkit-appearance:none;background:none;border:none;color:var(--color-primary-darker);margin:0;padding:0;position:relative}.tour_guide_anchor_text_area{border:none;border-radius:3px;font-family:Courier,monospace;font-size:11pt;font-weight:unset;padding:5px;width:unset}.Tooltip>.text-editor-field{display:flex;flex-flow:row nowrap;font-size:.95em;margin-bottom:1em;margin-top:1em}.Tooltip>.text-editor-field>label{color:revert;font-size:.95em;padding-bottom:.8em!important;padding-top:1.2em!important;width:10em!important}.Tooltip>.text-editor-field>input,.Tooltip>.text-editor-field>select,.Tooltip>.text-editor-field>textarea{font-size:.95em}
.Tooltip{z-index:500};
