@charset "UTF-8";

.navigation.breadcrumbs {
    display: inline-block;
    padding: 0.5rem 1rem 0.5rem 3rem;
    margin: 2rem 2rem 2rem 0rem;
}

.navigation.breadcrumbs,
.navigation.breadcrumbs * {
    font-family: identity, sans;
    font-weight: 400;
    font-size: var(--menu-font-size);
}

.navigation.breadcrumbs .navigation-option a {
    text-decoration: none;
    color: unset;
}
