{% load static %}
{% if did_enter %}
<div class="navigation{% if options.classname %} {{ options.classname }}{% endif %}"
     {% if options.margin %}style="margin: {{ options.margin.as_common_value }}"{% endif %}
>
{% else %}
</div>
{% endif %}
